/*
 * Copyright © 2021-2024 NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of Nvidia Corporation and its affiliates
 * (the "Company") and all right, title, and interest in and to the software
 * product, including all associated intellectual property rights, are and
 * shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 */

#ifndef API_CLX_PLUGIN_RUNNER_H_
#define API_CLX_PLUGIN_RUNNER_H_

#include <stdint.h>
#include <stddef.h>
#include <stdbool.h>

#ifdef __cplusplus
    #define BEGIN_C_DECLS  extern "C" {
    #define END_C_DECLS    }
#else
    #define BEGIN_C_DECLS
    #define END_C_DECLS
#endif

#define CLX_PLUGIN_RUNNER_DEFAULT_SAMPLING_INTERVAL_US 1000
#define CLX_PLUGIN_RUNNER_DEFAULT_SAMPLING_INTERVAL_MS (CLX_PLUGIN_RUNNER_DEFAULT_SAMPLING_INTERVAL_US / 1000)
#define CLX_PLUGIN_RUNNER_DEFAULT_NUM_ITERATIONS 1000
#define CLX_PLUGIN_RUNNER_DEFAULT_DATA_ROOT "clx_plugin_runner_data_root"
#define CLX_PLUGIN_RUNNER_DEFAULT_ID "clx_plugin_runner_id"
#define CLX_PLUGIN_RUNNER_DEFAULT_TAG "clx_plugin_runner_tag"
#define CLX_PLUGIN_RUNNER_DEFAULT_BUSY_WAIT_SAMPLING false
#define CLX_PLUGIN_RUNNER_DEFAULT_MATCH_METHOD "wildcard"
#define CLX_PLUGIN_RUNNER_DEFAULT_BUFFER_SIZE (60 * 1024)
#define CLX_PLUGIN_RUNNER_ENDLESS_LOOP 0
#define CLX_PLUGIN_RUNNER_DEFAULT_FILE_SIZE 1024 * 1024  // 1MB


typedef struct clx_plugin_runner_params_t {
    char* so_lib_name;                  ///< Path to plugin library to be loaded
    char* custom_opts_fname;            ///< Path to file with additional options should be used
    bool use_ipc;                       ///< Flag if IPC mechanism should be used, disabled by default
    char* ipc_dir;                      ///< Path to directory for storing files used for IPC
    char* fluent_bit_config_dir;        ///< Path to directory with fluent-bit configuration files
    char* prometheus_endpoint;          ///< Prometheus endpoint
    char* prometheus_cset_dir;          ///< Prometheus cset directory
    unsigned long sample_time_us;       ///< Plugin sampling interval in microseconds
    bool busy_wait_sampling;            ///< Flag if busy wait sampling should be used instead of sleep
    unsigned long num_iters;            ///< Amount of iteration for collecting data
    unsigned long buffer_size;          ///< Buffer size for collecting data
    int log_level;                      ///< Debug log level in range [1 , 7]
    char* log_file;                     ///< Path to log file
    char* include_names_file;           ///< File with include names
    char* name_match;                   ///< Name match method. one of 'exact', 'wildcard' or 'regexp'
    bool set_logger;                    ///< Flag if logger should be set. Should be true when running this as main application
    bool log_iterations;                ///< Flag if logger should be set. Should be true when running this as main application
    bool use_file_write;                ///< Flag if writing to binary files, enabled by default
    char* data_root;                    ///< Path to folder for storing binary files
    unsigned long file_size;            ///< Binary files size in bytes
    char* enabled_exporters;            ///< Enabled exporters comma separated list
    char* disabled_exporters;           ///< Disabled exporters comma separated list
    char* otel_receiver;                ///< OpenTelemetry receiver
    char* remote_write_receiver;        ///< Prometheus remote write receiver
    bool exit_on_plugin_load_error;     ///< Flag if to immediate exit on failing to load clx plugin
} clx_plugin_runner_params_t;

typedef struct clx_api_param_t {
    const char* key;
    const char* value;
} clx_api_param_t;

struct clx_plugin_runner_context_t;

/**
 * @brief Creates and returns context for clx_plugin_runner
 *
 * @param runner_params pointer to a clx_plugin_runner_params_t structure with parameters to be applied
 * @param custom_params pointer to a clx_api_param_t structure with parameters to be applied
 *
 * @return Pointer to clx_plugin_runner_context_t context.
 */
struct clx_plugin_runner_context_t* clx_plugin_runner_init_context(const clx_plugin_runner_params_t* runner_params,
                                                                   const clx_api_param_t* custom_params);

/**
 * @brief Initializes loaded clx providers
 *
 * @param ctx pointer to a clx_plugin_runner_context_t object
 *
 * @return 1 on failure
 * @return 0 on success
 */
int clx_plugin_runner_start(struct clx_plugin_runner_context_t* tester_ctx);

/**
 * @brief Stops loaded clx providers
 *
 * @param ctx pointer to a clx_plugin_runner_context_t object
 *
 */
void clx_plugin_runner_stop(struct clx_plugin_runner_context_t* tester_ctx);

/**
 * @brief Destroys clx_plugin_runner
 *
 * @param ctx pointer to a clx_plugin_runner_context_t object
 */
void clx_plugin_runner_destroy_context(struct clx_plugin_runner_context_t* ctx);

/**
 * @brief Returns number of loaded schemas in clx_plugin_runner_context_t
 *
 * @param ctx pointer to a clx_plugin_runner_context_t object
 *
 * @return number of loaded clx provider's schemas
 */
size_t clx_plugin_runner_get_num_of_schemas(struct clx_plugin_runner_context_t* ctx);


typedef void(*on_data_callback_t)(int, size_t, const char*);

/**
 * @brief Collect a portion of data for passed schema index
 *
 * @param ctx pointer to a clx_plugin_runner_context_t object
 * @param source_index provider source index (same as clx_api cloned context index)
 * @param on_data_callback delegate to handle collected data buffer
 *
 * @return true on failure
 * @return false on success
 */
bool clx_plugin_runner_do_loop_iteration(struct clx_plugin_runner_context_t* ctx,
                                         int source_index,
                                         on_data_callback_t on_data_callback);

/**
 * @brief Collects a portions of data for all registered schemas for a configured amount of iterations
 *
 * @param ctx pointer to a clx_plugin_runner_context_t object
 * @param on_data_callback delegate to handle collected data buffer
 *
 * @return true on failure
 * @return false on success
 */
bool clx_plugin_runner_do_loop(struct clx_plugin_runner_context_t* ctx, on_data_callback_t on_data_callback);

/**
 * @brief Returns null-terminated string with schema description
 *
 * @param ctx pointer to a clx_plugin_runner_context_t object
 * @param schema_index index of registered schema
 * @param pretty flag is schema should has pretty indention or not
 *
 * @return NULL on failure
 * @return string with schema description, returned string should be deallocated with free()
 */
char* clx_plugin_runner_get_schema(struct clx_plugin_runner_context_t* ctx, int schema_index, bool pretty);

/**
 * @brief Returns null-terminated string with schema description
 *
 * @param ctx pointer to a clx_plugin_runner_context_t object
 * @param schema_index index of registered schema
 * @param pretty flag is schema should has pretty indention or not
 *
 * @return NULL on failure
 * @return string with schema description, returned string should be deallocated with free()
 */
char* clx_plugin_runner_get_schema_id(struct clx_plugin_runner_context_t* ctx, int schema_index);

/**
 * @brief Sets plugin library to be loaded
 *
 * @param ctx pointer to a clx_plugin_runner_context_t object
 * @param so_lib_name path to clx plugin library
 *
 * @return true on success
 * @return false on failure
 */
bool clx_plugin_runner_set_so_lib_name(struct clx_plugin_runner_context_t* ctx, const char* so_lib_name);

/**
 * @brief Sets file with additional options should be used
 *
 * @param ctx pointer to a clx_plugin_runner_context_t object
 * @param custom_opts_fname path to file with options
 *
 * @return true on success
 * @return false on failure
 */
bool clx_plugin_runner_set_opts_fname(struct clx_plugin_runner_context_t* ctx, const char* custom_opts_fname);

/**
 * @brief Points if IPC mechanism should be used, disabled by default
 *
 * @param ctx pointer to a clx_plugin_runner_context_t object
 * @param use_ipc bool flag
 *
 * @return true on success
 * @return false on failure
 */
bool clx_plugin_runner_set_use_ipc(struct clx_plugin_runner_context_t* ctx, bool use_ipc);

/**
 * @brief Sets directory for storing fules used for IPC
 *
 * @param ctx pointer to a clx_plugin_runner_context_t object
 * @param ipc_dir path to file with options
 *
 * @return true on success
 * @return false on failure
 */
bool clx_plugin_runner_set_ipc_dir(struct clx_plugin_runner_context_t* ctx, const char* ipc_dir);

/**
 * @brief Sets clx plugin sampling interval in milliseconds, default is 1 millisecond
 *
 * @param ctx pointer to a clx_plugin_runner_context_t object
 * @param sample_time_ms sampling interval in milliseconds
 *
 * @return true on success
 * @return false on failure
 */
bool clx_plugin_runner_set_sample_time_ms(struct clx_plugin_runner_context_t* ctx, unsigned long sample_time_ms);

/**
 * @brief Sets clx plugin sampling interval in microseconds, default is 1000 microseconds
 *
 * @param ctx pointer to a clx_plugin_runner_context_t object
 * @param sample_time_ms sampling interval in microseconds
 *
 * @return true on success
 * @return false on failure
 */
bool clx_plugin_runner_set_sample_time_us(struct clx_plugin_runner_context_t* ctx, unsigned long sample_time_us);

/**
 * @brief Sets clx plugin sampling timing to be pedantic, default is false
 *
 * @param ctx pointer to a clx_plugin_runner_context_t object
 * @param busy_wait_sampling flag if busy wait sampling should be used instead of sleep
 *
 * @return true on success
 * @return false on failure
 */
bool clx_plugin_runner_set_busy_wait_sampling(struct clx_plugin_runner_context_t* ctx, bool busy_wait_sampling);



/**
 * Set whether plugin runner should set logger instance
 *
 * @param ctx pointer to a clx_plugin_runner_context_t object
 * @param set_logger Whether or not to set the logger
 *
 * @return true on success
 * @return false on failure
 */
bool clx_plugin_runner_set_set_logger(struct clx_plugin_runner_context_t* ctx, bool set_logger);

/**
 * Set whether plugin runner should log iterations
 *
 * @param ctx pointer to a clx_plugin_runner_context_t object
 * @param log_iterations Whether or not to log iterations
 *
 * @return true on success
 * @return false on failure   
*/
bool clx_plugin_runner_set_log_iterations(struct clx_plugin_runner_context_t* ctx, bool log_iterations);

/**
 * Sets the FluentBit configuration directory for the plugin runner
 *
 * @param ctx pointer to a clx_plugin_runner_context_t object
 * @param fluentbit_config_dir The path to the FluentBit configuration directory
 *
 * @return true on success
 * @return false on failure
 */
bool clx_plugin_runner_set_fluentbit_config_dir(struct clx_plugin_runner_context_t* ctx, const char* fluentbit_config_dir);

/**
 * @brief Sets prometheus endpoint
 *
 * @param ctx pointer to a clx_plugin_runner_context_t object
 * @param prometheus_endpoint prometheus endpoint
 * @return true on success
 * @return false on failure
 */
bool clx_plugin_runner_set_prometheus_endpoint(struct clx_plugin_runner_context_t* ctx, const char* prometheus_endpoint);

/**
 * @brief Sets prometheus counter set directory
 *
 * @param ctx pointer to a clx_plugin_runner_context_t object
 * @param prometheus_endpoint prometheus counter set directory
 * @return true on success
 * @return false on failure
 */
bool clx_plugin_runner_set_prometheus_cset_dir(struct clx_plugin_runner_context_t* ctx, const char* prometheus_cset_dir);

/**
 * @brief Sets amount of iteration for collecting data, default value is 1000
 *
 * @param ctx pointer to a clx_plugin_runner_context_t object
 * @param num_iters number of iterations, 0 means to collect indefinitely
 *
 * @return true on success
 * @return false on failure
 */
bool clx_plugin_runner_set_num_iters(struct clx_plugin_runner_context_t* ctx, unsigned long num_iters);

/**
 * @brief Sets buffer size for collecting data, default value is 60KB
 *
 * @param ctx pointer to a clx_plugin_runner_context_t object
 * @param buffer_size buffer size in bytes
 *
 * @return true on success
 * @return false on failure
 */
bool clx_plugin_runner_set_buffer_size(struct clx_plugin_runner_context_t* ctx, unsigned long buffer_size);

/**
 * @brief Sets debug log level, ERROR log level is set by default
 *
 * @param ctx pointer to a clx_plugin_runner_context_t object
 * @param log_level debug log level, should be from 1 to 7
 *
 * @return true on success
 * @return false on failure
 */
bool clx_plugin_runner_set_log_level(struct clx_plugin_runner_context_t* ctx, int log_level);

/**
 * @brief Sets log file
 *
 * @param ctx pointer to a clx_plugin_runner_context_t object
 * @param file_name path to log file
 *
 * @return true on success
 * @return false on failure
 */
bool clx_plugin_runner_set_log_file(struct clx_plugin_runner_context_t* ctx, const char* file_name);

/**
 * @brief Enables writing to binary files, enabled by default
 *
 * @param ctx pointer to a clx_plugin_runner_context_t object
 * @param use_file_write, flag if file writing should be enabled or not
 *
 * @return true on success
 * @return false on failure
 */
bool clx_plugin_runner_set_use_file_write(struct clx_plugin_runner_context_t* ctx, bool use_file_write);

/**
 * @brief Sets include names for clx plugin
 *
 * @param ctx pointer to a clx_plugin_runner_context_t object
 * @param file file with include names
 *
 * @return true on success
 * @return false on failure
 */
bool clx_plugin_runner_set_include_names_file(struct clx_plugin_runner_context_t* ctx, const char* include_names_file);

/**
 * @brief Sets name match method for clx plugin
 *
 * @param ctx pointer to a clx_plugin_runner_context_t object
 * @param name_match name match method. one of 'exact', 'wildcard' or 'regexp'
 *
 * @return true on success
 * @return false on failure
 */
bool clx_plugin_runner_set_name_match(struct clx_plugin_runner_context_t* ctx, const char* name_match);

/**
 * @brief Sets data root, by default set to "clx_plugin_runner_data_root"
 *
 * @param ctx pointer to a clx_plugin_runner_context_t object
 * @param data_root path to data root
 *
 * @return true on success
 * @return false on failure
 */
bool clx_plugin_runner_set_data_root(struct clx_plugin_runner_context_t* ctx, const char* data_root);

/**
 * @brief Enables immediate exit on failing to load clx plugin, disabled by default
 *
 * @param ctx pointer to a clx_plugin_runner_context_t object
 * @param exit_on_plugin_load_error flag if immediate exit or not
 *
 * @return true on success
 * @return false on failure
 */
bool clx_plugin_runner_set_exit_on_plugin_load_error(struct clx_plugin_runner_context_t* ctx,
                                                     bool exit_on_plugin_load_error);

/**
 * @brief Set enabled providers list
 *
 * @param ctx pointer to a clx_plugin_runner_context_t object
 * @param enabled_providers comma separated string with enabled providers names
 * @param len length of enabled_providers
 *
 * @return true on success
 * @return false on failure
 */
bool clx_plugin_runner_set_enabled_providers(struct clx_plugin_runner_context_t* ctx, const char* enabled_providers, size_t len);

/**
 * @brief Set disabled providers list
 *
 * @param ctx pointer to a clx_plugin_runner_context_t object
 * @param disabled_providers comma separated string with disabled providers names
 * @param len length of disabled_providers
 *
 * @return true on success
 * @return false on failure
 */
bool clx_plugin_runner_set_disabled_providers(struct clx_plugin_runner_context_t* ctx, const char* disabled_providers, size_t len);

/**
 * @brief Set enabled exporters list
 *
 * @param ctx pointer to a clx_plugin_runner_context_t object
 * @param enabled_exporters comma separated string with enabled exporters names
 *
 * @return true on success
 * @return false on failure 
 */
bool clx_plugin_runner_set_disabled_exporters(struct clx_plugin_runner_context_t* ctx, const char* disabled_exporters);

/**
 * @brief Set disabled exporters list
 *
 * @param ctx pointer to a clx_plugin_runner_context_t object
 * @param disabled_exporters comma separated string with disabled exporters names
 *
 * @return true on success
 * @return false on failure
 */
bool clx_plugin_runner_set_enabled_exporters(struct clx_plugin_runner_context_t* ctx, const char* enabled_exporters);

/**
 * @brief Set OpenTelemetry receiver
 * 
 * @param ctx pointer to a clx_plugin_runner_context_t object
 * @param otel_receiver OpenTelemetry receiver
 * @return true on success
 * @return false on failure
 */
bool clx_plugin_runner_set_open_telemetry_receiver(struct clx_plugin_runner_context_t* ctx, const char* otel_receiver);

/**
 * @brief Set remote write receiver
 * 
 * @param ctx pointer to a clx_plugin_runner_context_t object
 * @param otel_receiver remote write receiver
 * @return true on success
 * @return false on failure
 */
bool clx_plugin_runner_set_remote_write_receiver(struct clx_plugin_runner_context_t* ctx, const char* remote_write_receiver);

/**
 * @brief Set binary files size. Actual size may be adjusted to align with buffer granularity.
 *
 * @param ctx pointer to a clx_plugin_runner_context_t object
 * @param file_size file size in bytes.
 * @return true on success
 * @return false on failure
 */
bool clx_plugin_runner_set_file_size(struct clx_plugin_runner_context_t* ctx, unsigned long file_size);

/**
 * @brief Add a key-value entry to plugin options
 *
 * @param ctx pointer to a clx_plugin_runner_context_t object
 * @param key option key
 * @param value option value
 *
 * @return true on success
 * @return false on failure
 */
bool clx_plugin_runner_add_plugin_option(struct clx_plugin_runner_context_t* ctx, const char* key, const char* value);

/**
 * @brief Initiates plugin runner's main loop termination
 *
 * @param ctx pointer to a clx_plugin_runner_context_t object
 *
 * @return true on failure
 * @return false on success
 */
bool clx_plugin_runner_initiate_loop_termination(struct clx_plugin_runner_context_t* ctx);


#endif  // API_CLX_PLUGIN_RUNNER_H_
